# coding=utf-8
"""
"""
from __future__ import absolute_import

import jinja2
from abilian.services.preferences import preferences

from .panels.sbe_notifications import SbeNotificationsPanel


def register_plugin(app):
    app.register_jinja_loaders(jinja2.PackageLoader(__name__, 'templates'))
    preferences.register_panel(SbeNotificationsPanel(), app)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

import logging
import time
from datetime import datetime
from os.path import dirname, join

import sqlalchemy as sa
from blinker import ANY
from flask import current_app
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, \
    String, Unicode, UniqueConstraint, and_
from sqlalchemy.event import listens_for
from sqlalchemy.orm import backref, relation, relationship
from sqlalchemy.orm.attributes import OP_APPEND, OP_REMOVE

from abilian.core.entities import Entity
from abilian.core.extensions import db
from abilian.core.models import NOT_AUDITABLE, SEARCHABLE
from abilian.core.models.blob import Blob
from abilian.core.models.subjects import Group, User
from abilian.i18n import _l
from abilian.sbe.apps.documents.models import Folder
from abilian.sbe.apps.documents.repository import repository
from abilian.services.indexing import indexable_role
from abilian.services.security import Manager as MANAGER
from abilian.services.security import Reader as READER
from abilian.services.security import Writer as WRITER
from abilian.services.security import READ, WRITE, Admin, Permission, Role, \
    RoleType, security

from . import signals

logger = logging.getLogger(__name__)

MEMBER = Role('member', label=_l(u'role_member'), assignable=False)
VALID_ROLES = frozenset([READER, WRITER, MANAGER, MEMBER])


class Membership(db.Model):
    """Represents the membership of someone in a community.
    """
    __tablename__ = 'community_membership'

    id = Column(Integer, primary_key=True)

    user_id = Column(ForeignKey('user.id'), index=True, nullable=False)
    user = relationship(
        User,
        lazy='joined',
        backref=backref('communautes_membership', lazy='select'))

    community_id = Column(
        ForeignKey('community.id'), index=True, nullable=False)
    community = relationship('Community', lazy='joined')

    role = Column(RoleType())  # should be either 'member' or 'manager'

    __table_args__ = (UniqueConstraint('user_id', 'community_id'),)

    def __repr__(self):
        return u'<Membership user={}, community={}, role={}>'.format(
            repr(self.user), repr(self.community),
            str(self.role)).encode('utf-8')


def community_content(cls):
    """Class decorator to mark models considered as community content.

    This is required for proper indexation.
    """
    cls.is_community_content = True

    def community_slug(self):
        return self.community and self.community.slug

    cls.community_slug = property(community_slug)
    index_to = cls.__indexation_args__.setdefault('index_to', ())
    index_to += (('community_slug', ('community_slug',)),)
    cls.__indexation_args__['index_to'] = index_to

    def _indexable_roles_and_users(self):
        if not self.community:
            return []
        return indexable_roles_and_users(self.community)

    cls._indexable_roles_and_users = property(_indexable_roles_and_users)
    return cls


def indexable_roles_and_users(community):
    """Mixin to use to replace Entity._indexable_roles_and_users.

    Will be removed when communities are upgraded to use standard role based
    access (by setting permissions and using security service).
    """
    return u' '.join(indexable_role(user) for user in community.members)


class Community(Entity):
    """Ad-hoc objects that hold properties about a community.
    """
    __indexation_args__ = {}
    __indexation_args__.update(Entity.__indexation_args__)
    index_to = __indexation_args__.setdefault('index_to', ())
    index_to += (('id', ('id', 'community_id')),)
    __indexation_args__['index_to'] = index_to
    del index_to

    is_community_content = True

    # : A public description.
    description = Column(
        Unicode(500), default=u"", nullable=False, info=SEARCHABLE)

    #: An image or logo for this community.
    image_id = Column(ForeignKey(Blob.id), index=True)
    image = relationship(Blob, lazy='joined')

    #: The root folder for this community.
    folder_id = Column(
        Integer,
        ForeignKey(Folder.id, use_alter=True, name='fk_community_root_folder'),
        unique=True)
    folder = relation(
        Folder,
        single_parent=True,  # required for delete-orphan
        primaryjoin=(folder_id == Folder.id),
        cascade='all, delete-orphan',
        backref=backref('_community', lazy='select', uselist=False))

    #: The group this community is linked to, if any. Memberships are then
    #: reflected
    group_id = Column(ForeignKey(Group.id), nullable=True, unique=True)
    group = relation(Group, foreign_keys=group_id, lazy='select')

    #: Memberships for this community.
    memberships = relationship(Membership, cascade="all, delete-orphan")

    #: direct access to :class:`User` members
    members = relationship(
        User,
        secondary=Membership.__table__,
        viewonly=True,
        backref=backref('communities', lazy='select', viewonly=True))

    #: Number of members in this community.
    membership_count = Column(
        Integer, default=0, nullable=False, info=NOT_AUDITABLE)

    #: Number of documents in this community.
    document_count = Column(
        Integer, default=0, nullable=False, info=NOT_AUDITABLE)

    #: Last time something happened in this community
    last_active_at = Column(
        DateTime, default=datetime.utcnow, nullable=False, info=NOT_AUDITABLE)

    # Various features for this community:

    #: True if this community has a document management space
    has_documents = Column(Boolean, nullable=False, default=True)

    #: True if this community has a wiki
    has_wiki = Column(Boolean, nullable=False, default=True)

    #: True if this community has a forum
    has_forum = Column(Boolean, nullable=False, default=True)

    #: One of 'participative' or 'informative
    type = Column(String(20), nullable=False, default="informative")

    #: One of 'secret', 'public', 'open' (currently not used)
    visibility = Column(String(20), nullable=False, default="secret")

    #: Used for segmenting communities. Not used currently.
    category = Column(String(20), nullable=False, default="")

    #: True if regular members can send stuff by email to all members
    # Not used anymore.
    members_can_send_by_email = Column(Boolean, nullable=False, default=False)

    def __init__(self, **kw):
        self.has_documents = True
        self.membership_count = 0
        self.document_count = 0
        self.members_can_send_by_email = False
        Entity.__init__(self, **kw)
        if self.has_documents and not self.folder:
            # FIXME: this should be done in documents by using signals
            name = self.name
            if not name:
                # during creation, we may have to provide a temporary name for
                # subfolder, we don't want empty names on folders since they must be
                # unique among siblings
                name = u'{}_{}-{}'.format(self.__class__.__name__,
                                          str(self.id), time.asctime())
            self.folder = repository.root_folder.create_subfolder(name)
            # if not self.group:
            #   self.group = Group(name=self.name)

        if not self.image:
            fn = join(dirname(__file__), "views/data", "community.png")
            self.image = Blob(open(fn, 'rb').read())

    @property
    def has_calendar(self):
        config = current_app.config
        return bool(config.get("ENABLE_CALENDAR"))

    def rename(self, name):
        self.name = name
        if self.folder:
            # FIXME: use signals
            self.folder.name = name

    def get_memberships(self, role=None):
        M = Membership
        memberships = M.query.filter(M.community_id == self.id)
        if role:
            memberships = memberships.filter(M.role == role)
        return memberships.all()

    def set_membership(self, user, role):
        """
        Add a member with the given role, or set the role of an existing
        member.
        """
        assert isinstance(user, User)
        role = Role(role)

        if role not in VALID_ROLES:
            raise ValueError("Invalid role: {}".format(role))

        session = sa.orm.object_session(self) or db.session()
        is_new = True
        M = Membership
        membership = session.query(M) \
            .filter(and_(M.user_id == user.id,
                         M.community_id == self.id)) \
            .first()

        if not membership:
            membership = Membership(community=self, user=user, role=role)
            session.add(membership)
            self.membership_count += 1
        else:
            is_new = False
            membership.role = role

        signals.membership_set.send(self, membership=membership, is_new=is_new)

    def remove_membership(self, user):
        M = Membership
        membership = M.query \
            .filter(and_(M.user_id == user.id, M.community_id == self.id)) \
            .first()
        if not membership:
            raise KeyError("User {} is not a member of community {}".format(
                user, self))

        db.session.delete(membership)
        self.membership_count -= 1
        signals.membership_removed.send(self, membership=membership)

    def update_roles_on_folder(self):
        if self.folder:
            self.ungrant_all_roles_on_folder()
            for membership in self.memberships:
                user = membership.user
                role = membership.role
                if role == MANAGER:
                    security.grant_role(user, MANAGER, self.folder)
                else:
                    if self.type == 'participative':
                        security.grant_role(user, WRITER, self.folder)
                    else:
                        security.grant_role(user, READER, self.folder)

    def ungrant_all_roles_on_folder(self):
        if self.folder:
            role_assignments = security.get_role_assignements(self.folder)
            for principal, role in role_assignments:
                security.ungrant_role(principal, role, self.folder)

    def get_role(self, user):
        """Returns the given user's role in this community.
        """
        M = Membership
        membership = current_app.db.session() \
            .query(M.role) \
            .filter(and_(M.community_id == self.id,
                         M.user_id == user.id)) \
            .first()

        return membership.role if membership else None

    def has_member(self, user):
        return self.get_role(user) is not None

    def has_permission(self, user, permission):
        if not isinstance(permission, Permission):
            assert isinstance(permission, str)
            permission = Permission(permission)

        if user.has_role(Admin):
            return True
        role = self.get_role(user)
        if role == MANAGER:
            return True
        if role == MEMBER and permission in (READ, WRITE):
            return True
        return False

    def touch(self):
        self.last_active_at = datetime.utcnow()

    @property
    def _indexable_roles_and_users(self):
        return indexable_roles_and_users(self)


def CommunityIdColumn():
    return Column(
        ForeignKey(Community.id),
        nullable=False,
        info=SEARCHABLE | dict(index_to=(('community_id', ('community_id',)),)))


# Handlers to keep community/group members in sync
#
_PROCESSED_ATTR = '__sbe_community_group_sync_processed__'


@signals.membership_set.connect_via(ANY)
def _membership_added(sender, membership, is_new):
    if not is_new:
        return

    if getattr(membership.user, _PROCESSED_ATTR, False) is OP_APPEND:
        return

    if sender.group and membership.user not in sender.group.members:
        logger.debug('_membership_added(%r, %r, %r) user: %r', sender,
                     membership, is_new, membership.user)
        setattr(membership.user, _PROCESSED_ATTR, OP_APPEND)
        sender.group.members.add(membership.user)


@signals.membership_removed.connect_via(ANY)
def membership_removed(sender, membership):
    if getattr(membership.user, _PROCESSED_ATTR, False) is OP_REMOVE:
        return

    if sender.group and membership.user in sender.group.members:
        logger.debug('_membership_removed(%r, %r) user: %r', sender, membership,
                     membership.user)
        setattr(membership.user, _PROCESSED_ATTR, OP_REMOVE)
        sender.group.members.discard(membership.user)


@listens_for(Community.members, 'append')
@listens_for(Community.members, 'remove')
def _on_member_change(community, user, initiator):
    group = community.group
    if not group:
        return

    logger.debug('_on_member_change(%r, %r, op=%r)', community, user,
                 initiator.op)

    if getattr(user, _PROCESSED_ATTR, False) is initiator.op:
        return

    setattr(user, _PROCESSED_ATTR, initiator.op)

    if initiator.op is OP_APPEND:
        if user not in group.members:
            group.members.add(user)

    elif initiator.op is OP_REMOVE:
        if user in group.members:
            group.members.discard(user)


@listens_for(Community.group, 'set', active_history=True)
def _on_linked_group_change(community, value, oldvalue, initiator):
    if value == oldvalue:
        return

    logger.debug('_on_linked_group_change(%r, %r, %r)', community, value,
                 oldvalue)

    if oldvalue is not None and oldvalue.members:
        logger.debug('_on_linked_group_change(%r, %r, %r): oldvalue clear()',
                     community, value, oldvalue)
        oldvalue.members.clear()

    members = set(community.members)
    if value is not None and value.members != members:
        logger.debug('_on_linked_group_change(%r, %r, %r): set value.members',
                     community, value, oldvalue)
        value.members = members


def _safe_get_community(group):
    session = sa.orm.object_session(group)
    if not session:
        return None

    with session.no_autoflush:
        try:
            return session.query(Community) \
                .filter(Community.group == group) \
                .options(sa.orm.joinedload(Community.group),
                         sa.orm.joinedload(Community.members)) \
                .one()
        except sa.orm.exc.NoResultFound:
            return None


@listens_for(Group.members, 'append')
@listens_for(Group.members, 'remove')
def _on_group_member_change(group, user, initiator):
    community = _safe_get_community(group)

    if not community:
        return

    op = initiator.op
    if getattr(user, _PROCESSED_ATTR, False) is op:
        return

    is_present = user in community.members
    setattr(user, _PROCESSED_ATTR, op)
    logger.debug('_on_group_member_change(%r, %r, op=%r) community: %r', group,
                 user, initiator.op, community)

    if ((op is OP_APPEND and is_present) or
        (op is OP_REMOVE and not is_present)):
        return

    if op is OP_APPEND:
        community.set_membership(user, MEMBER)

    elif op is OP_REMOVE:
        community.remove_membership(user)


@listens_for(Group.members, 'set', active_history=True)
def _on_group_members_replace(group, value, oldvalue, initiator):
    if value == oldvalue:
        return

    community = _safe_get_community(group)
    if not community:
        return

    members = set(community.members)
    logger.debug('_on_group_members_replace(%r, %r, %r) community: %r', group,
                 value, oldvalue, community)

    for u in members - value:
        if getattr(u, _PROCESSED_ATTR, False) is OP_REMOVE:
            continue
        setattr(u, _PROCESSED_ATTR, OP_REMOVE)
        community.remove_membership(u)

    for u in value - members:
        if getattr(u, _PROCESSED_ATTR, False) is OP_APPEND:
            continue
        setattr(u, _PROCESSED_ATTR, OP_APPEND)
        community.set_membership(u, MEMBER)



# coding=utf-8
"""
Set up the markdown converter. Add extensions here (for now).
"""
from __future__ import absolute_import, print_function

import markdown
from flask import url_for
from markdown.extensions.wikilinks import WikiLinkExtension, WikiLinks
from markdown.util import etree

__all__ = ['convert']


class UrlBuilder(object):

    def __init__(self, page):
        self.page = page

    def build(self, label, base, end):
        label = label.strip()
        return url_for(
            ".page", community_id=self.page.community.slug, title=label)


def convert(page, text):
    build_url = UrlBuilder(page).build
    extension = SBEWikiLinkExtension([('build_url', build_url)])
    ctx = {}
    ctx['extensions'] = [extension, 'toc']
    ctx['output_format'] = 'html5'
    md = markdown.Markdown(**ctx)
    return md.convert(text)


class SBEWikiLinkExtension(WikiLinkExtension):

    def extendMarkdown(self, md, md_globals):
        self.md = md

        # append to end of inline patterns
        WIKILINK_RE = r'\[\[(.*?)\]\]'
        wikilinkPattern = SBEWikiLinks(WIKILINK_RE, self.getConfigs())
        wikilinkPattern.md = md
        md.inlinePatterns.add('wikilink', wikilinkPattern, "<not_strong")


class SBEWikiLinks(WikiLinks):

    def handleMatch(self, m):
        from .forms import page_exists
        if m.group(2).strip():
            base_url, end_url, html_class = self._getMeta()
            label = m.group(2).strip()
            url = self.config['build_url'](label, base_url, end_url)
            a = etree.Element('a')
            a.text = label
            a.set('href', url)
            if html_class:
                if page_exists(label):
                    a.set('class', html_class)
                else:
                    a.set('class', html_class + ' new')
        else:
            a = ''
        return a



from __future__ import absolute_import, print_function

from flask import render_template

# TEMP
ROOT = "http://localhost:5000/cmis/atompub"
XML_HEADER = "<?xml version='1.0' encoding='UTF-8'?>\n"


class Feed(object):

    def __init__(self, object, collection):
        self.object = object
        self.collection = collection

    def to_xml(self, **options):
        ctx = {
            'ROOT': ROOT,
            'object': self.object,
            'collection': self.collection,
            'to_xml': to_xml
        }
        return render_template("cmis/feed.xml", **ctx)


class Entry(object):

    def __init__(self, obj):
        self.obj = obj

    def to_xml(self, **options):
        ctx = {
            'ROOT': ROOT,
            'folder': self.obj,
            'document': self.obj,
            'options': options,
            'to_xml': to_xml
        }

        if self.obj.sbe_type == 'cmis:folder':
            result = render_template("cmis/folder.xml", **ctx)
        elif self.obj.sbe_type == 'cmis:document':
            result = render_template("cmis/document.xml", **ctx)
        else:
            raise Exception("Unknown base object type: %s" % self.obj.sbe_type)

        if 'no_xml_header' not in options:
            result = XML_HEADER + result

        return result


def to_xml(obj, **options):
    entry = Entry(obj)
    return entry.to_xml(**options)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

import imghdr

import PIL
import sqlalchemy as sa
from flask import request
from flask_babel import gettext as _
from flask_babel import lazy_gettext as _l
from six import text_type
from wtforms.fields import BooleanField, StringField, TextAreaField
from wtforms.validators import ValidationError, data_required, optional

from abilian.core.models.subjects import Group
from abilian.web.forms import Form
from abilian.web.forms.fields import FileField, Select2Field
from abilian.web.forms.validators import length
from abilian.web.forms.widgets import BooleanWidget, ImageInput, TextArea

from .models import Community


def strip(s):
    return s.strip()


def _group_choices():
    m_prop = Group.members.property
    membership = m_prop.secondary
    query = Group.query.session.query(
        Group.id, Group.name,
        Community.name.label('community'),
        sa.sql.func.count(membership.c.user_id).label('members_count'))
    query = query.outerjoin(m_prop.secondary, m_prop.primaryjoin) \
        .outerjoin(Community, Community.group.property.primaryjoin) \
        .group_by(Group.id, Group.name, Community.name) \
        .order_by(sa.sql.func.lower(Group.name)) \
        .autoflush(False)
    choices = [(u'', u'')]

    for g in query:
        label = u'{} ({:d} membres)'.format(g.name, g.members_count)
        if g.community:
            label += u' — Communauté: {}'.format(g.community)
        choices.append((text_type(g.id), label))

    return choices


class CommunityForm(Form):
    name = StringField(label=_l(u"Name"), validators=[data_required()])
    description = TextAreaField(
        label=_l(u"Description"),
        validators=[data_required(), length(max=500)],
        widget=TextArea(resizeable="vertical"))

    linked_group = Select2Field(
        label=_l(u'Linked to group'),
        description=_l(
            u'Manages a group of users through this community members.'),
        choices=_group_choices)

    image = FileField(
        label=_l('Image'),
        widget=ImageInput(width=65, height=65),
        validators=[optional()])

    type = Select2Field(label=_(u"Type"), validators=[data_required()],
                        filters=(strip,),
                        choices=[(_l(u'informative'), 'informative'),
                                 (_l(u'participative'), 'participative')])

    has_documents = BooleanField(
        label=_l(u"Has documents"), widget=BooleanWidget(on_off_mode=True))
    has_wiki = BooleanField(
        label=_l(u"Has a wiki"), widget=BooleanWidget(on_off_mode=True))
    has_forum = BooleanField(
        label=_l(u"Has a forum"), widget=BooleanWidget(on_off_mode=True))

    def validate_name(self, field):
        name = field.data = field.data.strip()

        if name and field.object_data:
            # form is bound to an existing object, name is not empty
            if name != field.object_data:
                # name changed: check for duplicates
                if Community.query.filter(Community.name == name).count() > 0:
                    raise ValidationError(_(u"A community with this name already exists"))

    def validate_description(self, field):
        field.data = field.data.strip()

    # FIXME: code duplicated from the user edit form (UserProfileForm).
    # Needs to be refactored.
    def validate_image(self, field):
        data = request.form.get('image')
        if not data:
            return

        data = field.data
        filename = data.filename
        valid = any(map(filename.lower().endswith, ('.png', '.jpg', '.jpeg')))

        if not valid:
            raise ValidationError(_(u'Only PNG or JPG image files are accepted'))

        img_type = imghdr.what('ignored', data.read())

        if img_type not in ('png', 'jpeg'):
            raise ValidationError(_(u'Only PNG or JPG image files are accepted'))

        data.seek(0)
        try:
            # check this is actually an image file
            im = PIL.Image.open(data)
            im.load()
        except:
            raise ValidationError(_(u'Could not decode image file'))

        data.seek(0)
        field.data = data



"""
NOTE: this code is a legacy from the early days of the application,
and currently not used.
"""

from __future__ import absolute_import, print_function

import json

from flask import Blueprint, g, make_response, request
from flask_login import login_required

from abilian.core.extensions import db
from abilian.core.models.subjects import Group, User
from abilian.core.util import get_params

from .models import Message

__all__ = ['restapi']

restapi = Blueprint('restapi', __name__, url_prefix="/api")

# Util


def make_json_response(obj, response_code=200):
    if isinstance(obj, list):
        obj = [x.to_dict() if hasattr(x, 'to_dict') else x for x in obj]

    if hasattr(obj, 'to_json'):
        response = make_response(obj.to_json(), response_code)
    elif hasattr(obj, 'to_dict'):
        response = make_response(json.dumps(obj.to_dict()), response_code)
    else:
        response = make_response(json.dumps(obj), response_code)
    response.mimetype = 'application/json'
    return response


#
# Users
#


# [POST] /api/users/USER_ID	Create User Profile
@restapi.route("/users", methods=['POST'])
@login_required
def create_user():
    d = get_params(User.__editable__)
    user = User(**d)
    db.session.add(user)
    db.session.commit()
    return make_json_response(user, 201)


# [GET] /api/users	List Users in Your Organization
@restapi.route("/users")
@login_required
def list_users():
    # l = [ u.to_dict() for u in User.query.all() ]
    l = list(User.query.all())
    return make_json_response(l)


# [GET] /api/users/USER_ID	View User Profile
@restapi.route("/users/<int:user_id>")
@login_required
def get_user(user_id):
    user = User.query.get(user_id)
    return make_json_response(user)


# [GET] /api/users/USER_ID/messages	View Stream of Messages by User
@restapi.route("/users/<int:user_id>/messages")
@login_required
def user_stream(user_id):
    user = User.query.get(user_id)
    messages = Message.query.by_creator(user).all()
    # messages = list(user.messages)
    return make_json_response(messages)


# [PUT] /api/users/USER_ID	Update User Profile
@restapi.route("/users/<int:user_id>", methods=['PUT'])
@login_required
def update_user(user_id):
    user = User.query.get(user_id)
    d = get_params(User.__editable__)
    user.update(d)
    db.session.commit()
    return make_json_response(user)


# [DELETE] /api/users/USER_ID	Deactivate a User
@restapi.route("/users/<int:user_id>", methods=['DELETE'])
@login_required
def delete_user(user_id):
    user = User.query.get(user_id)
    db.session.delete(user)
    db.session.commit()
    return make_response("", 204)


#
# Social graph: following
#


# [GET] /api/users/USER_ID/followers	View Followers of User
@restapi.route("/users/<int:user_id>/followers")
@login_required
def get_followers(user_id):
    user = User.query.get(user_id)
    followers = list(user.followers)
    return make_json_response(followers)


# [GET] /api/users/USER_ID/followees	View List of Users Being Followed
@restapi.route("/users/<int:user_id>/followees")
@login_required
def get_followees(user_id):
    user = User.query.get(user_id)
    followees = list(user.followees)
    return make_json_response(followees)


# [POST] /api/users/USER_ID/followers	Follow a User
@restapi.route("/users/<int:user_id>/followers", methods=['POST'])
@login_required
def follow(user_id):
    user = User.query.get(user_id)
    g.user.follow(user)
    db.session.commit()
    return make_json_response("", 204)


# [DELETE] /api/users/USER_ID/followers/CONTACT_USER_ID	Unfollow a User
@restapi.route(
    "/users/<int:user_id>/followers/<int:contact_user_id>", methods=['DELETE'])
@login_required
def unfollow(user_id, contact_user_id):
    user = User.query.get(user_id)
    g.user.unfollow(user)
    db.session.commit()
    return make_json_response("", 204)


#
# Social graph: groups
#


# [GET] /api/groups	Listing All Groups
@restapi.route("/groups")
@login_required
def list_groups():
    l = list(Group.query.all())
    return make_json_response(l)


# [GET] /api/groups/GROUP_ID	Show a Single Group
@restapi.route("/groups/<int:group_id>")
@login_required
def get_group(group_id):
    group = Group.query.get(group_id)
    return make_json_response(group)


# [GET] /api/groups/GROUP_ID/members	Listing Members of a Group
@restapi.route("/groups/<int:group_id>/members")
@login_required
def get_group_members(group_id):
    group = Group.query.get(group_id)
    return make_json_response(group.members)


# [GET] /api/group_memberships	Listing Group Memberships


# [POST] /api/groups	Create a Group
@restapi.route("/groups", methods=['POST'])
@login_required
def create_group():
    d = get_params(Group.__editable__)
    group = Group(**d)
    db.session.add(group)
    db.session.commit()
    return make_json_response(group, 201)


# [PUT] /api/groups/GROUP_ID	Updating Existing Group

# [DELETE] /api/groups/GROUP_ID/archive	Archiving a Group

# [DELETE] /api/groups/GROUP_ID	Destroy an Archived Message

#
# Messages
#


# [POST] /api/messages	Creating New Messages
@restapi.route("/messages", methods=['POST'])
@login_required
def create_message():
    d = get_params(Message.__editable__)
    message = Message(creator_id=g.user.id, **d)
    db.session.add(message)
    db.session.commit()
    return make_json_response(message, 201)


# [GET] /api/messages	Reading Stream Messages
@restapi.route("/messages")
@login_required
def get_messages():
    messages = list(Message.query.all())
    return make_json_response(messages)


# [GET] /api/messages/MESSAGE_ID	Read a Single Stream Message
@restapi.route("/messages/<int:message_id>")
@login_required
def get_message(message_id):
    message = Message.query.get(message_id)
    return make_json_response(message)


# [PUT] /api/messages/MESSAGE_ID	Updating Existing Messages
@restapi.route("/messages/<int:message_id>", methods=['PUT'])
@login_required
def update_message(message_id):
    message = Message.query.get(message_id)
    d = get_params(["content"])
    message.update(d)
    db.session.commit()
    return make_json_response(message)


# [DELETE] /api/messages/MESSAGE_ID	Destroy an existing message
@restapi.route("/messages/<int:message_id>", methods=['DELETE'])
@login_required
def delete_message(message_id):
    message = Message.query.get(message_id)
    db.session.delete(message)
    db.session.commit()
    return make_response("", 204)


#
# Likes
#
# TODO: use an "objects" namespace instead to make it generic?


# [POST] /api/messages/MESSAGE_ID/likes	Liking a Message
@restapi.route("/messages/<int:message_id>/likes", methods=['POST'])
@login_required
def like_message(message_id):
    pass


# [POST] /api/comments/COMMENT_ID/likes/LIKES_ID	Liking a Comment
@restapi.route("/comments/<int:comment_id>/likes", methods=['POST'])
@login_required
def like_comment(comment_id):
    pass


# [DELETE] /api/messages/MESSAGE_ID/likes/LIKES_ID	Un-liking a Message
@restapi.route(
    "/messages/<int:message_id>/likes/<int:like_id>", methods=['DELETE'])
@login_required
def unlike_message(message_id, like_id):
    pass


# [DELETE] /api/comments/COMMENT_ID/likes/LIKES_ID
@restapi.route(
    "/comments/<int:comment_id>/likes/<int:like_id>", methods=['DELETE'])
@login_required
def unlike_comment(comment_id, like_id):
    pass


#
# Search
#


# [GET] /api/messages/search	Searching Messages
@restapi.route("/search/messages")
@login_required
def search_messages():
    q = request.args.get("q")
    if not q:
        return make_json_response([])
    messages = list(Message.search_query(q).all())
    return make_json_response(messages)


# [GET] /api/users/search	Search Users in Your Company
@restapi.route("/search/users")
@login_required
def search_users():
    q = request.args.get("q")
    if not q:
        return make_json_response([])
    users = list(User.search_query(q).all())
    return make_json_response(users)


#
# Activity Streams
#
@restapi.route("/feed")
@login_required
def get_feed():
    pass



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

from abilian.sbe.apps.communities.models import VALID_ROLES
from abilian.sbe.apps.communities.signals import membership_removed, \
    membership_set
from abilian.services.security import Manager, Reader, Writer, security

from .search import reindex_tree


@membership_set.connect
def new_community_member(community, membership, is_new, **kwargs):
    if not community.folder:
        return

    role = membership.role
    user = membership.user
    local_role = Writer if community.type == 'participative' else Reader
    if role == Manager:
        local_role = Manager

    current_roles = set(
        security.get_roles(user, community.folder, no_group_roles=True))
    current_roles &= VALID_ROLES  # ensure we don't remove roles not managed
    # by us

    for role_to_ungrant in current_roles - {local_role}:
        security.ungrant_role(user, role_to_ungrant, community.folder)

    if local_role not in current_roles:
        security.grant_role(user, local_role, community.folder)

    reindex_tree(community.folder)


@membership_removed.connect
def remove_community_member(community, membership, **kwargs):
    if not community.folder:
        return

    user = membership.user
    roles = set(security.get_roles(user, community.folder, no_group_roles=True))
    roles &= VALID_ROLES  # ensure we don't remove roles not managed by us
    for role in roles:
        security.ungrant_role(user, role, community.folder)

    reindex_tree(community.folder)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

import sqlalchemy as sa
from flask_script import Manager

from . import tasks
from .models import Document

manager = Manager(
    description='SBE documents actions', help='SBE documents actions')


@manager.command
def antivirus():
    """Schedule documents to antivirus scan.
    """

    documents = Document.query \
        .filter(Document.content_blob != None) \
        .options(sa.orm.noload('creator'),
                 sa.orm.noload('owner'),
                 sa.orm.joinedload('content_blob'))

    total = 0
    count = 0
    for d in documents.yield_per(1000):
        total += 1
        meta = d.content_blob.meta
        if 'antivirus' not in meta and 'antivirus_task' not in meta:
            tasks.antivirus_scan.delay(d.id)
            count += 1

    print('{count}/{total} documents scheduled'.format(
        count=count, total=total))



# coding=utf-8
"""
Calendar module.
"""
from __future__ import absolute_import

from abilian.sbe.extension import sbe


def register_plugin(app):
    sbe.init_app(app)

    from .views import blueprint
    from .actions import register_actions

    blueprint.record_once(register_actions)
    app.register_blueprint(blueprint)



from __future__ import absolute_import, print_function

from os.path import dirname

from lxml import etree

from ..webdav.constants import DAV_PROPS
from ..webdav.xml import MultiStatus, Propfind


def test_propfind_sample1():
    xml = open(dirname(__file__) + "/data/propfind1.xml", 'rb').read()
    propfind = Propfind(xml)
    assert propfind.mode == 'prop'


def test_propfind_sample2():
    xml = open(dirname(__file__) + "/data/propfind2.xml", 'rb').read()
    propfind = Propfind(xml)
    assert propfind.mode == 'prop'


def test_propfind_sample3():
    xml = open(dirname(__file__) + "/data/propfind3.xml", 'rb').read()
    propfind = Propfind(xml)
    assert propfind.mode == 'allprop'


def test_empty_multistatus():
    m = MultiStatus()
    result = m.to_string()

    # Check XML is weel-formed
    etree.fromstring(result)


def test_multistatus():

    class Obj(object):
        pass

    obj = Obj()
    obj.name = "some name"
    obj.is_folder = True
    m = MultiStatus()
    m.add_response_for("http://example.com/", obj, DAV_PROPS)
    result = m.to_string()

    # Check XML is weel-formed
    etree.fromstring(result)



# coding=utf-8
from __future__ import absolute_import, print_function

from flask import url_for as url_for_orig
from flask import g
from flask_babel import lazy_gettext as _l
from flask_login import current_user

from abilian.web.action import Action, Endpoint, actions
from abilian.web.nav import NavItem

__all__ = ['register_actions']


def url_for(endpoint, **kw):
    return url_for_orig(endpoint, community_id=g.community.slug, **kw)


class CommunityEndpoint(Endpoint):

    def get_kwargs(self):
        kwargs = super(CommunityEndpoint, self).get_kwargs()
        kwargs['community_id'] = g.community.slug
        return kwargs


class CommunityTabAction(Action):

    Endpoint = CommunityEndpoint

    def url(self, context=None):
        if self._url:
            return Action.url(self)

        endpoint = self.endpoint
        if endpoint:
            return endpoint
        else:
            return url_for("%s.index" % self.name)

    def is_current(self):
        return g.current_tab == self.name


_actions = (
    # Navigation
    NavItem(
        'section',
        'communities',
        title=_l(u'Communities'),
        url=lambda context: url_for_orig('communities.index'),
        condition=lambda ctx: current_user.is_authenticated),
    # Tabs
    CommunityTabAction('communities:tabs', 'wall', _l(u'Activities')),
    CommunityTabAction(
        'communities:tabs',
        'documents',
        _l(u'Documents'),
        condition=lambda ctx: g.community.has_documents),
    CommunityTabAction(
        'communities:tabs',
        'wiki',
        _l(u'Wiki'),
        condition=lambda ctx: g.community.has_wiki),
    CommunityTabAction(
        'communities:tabs',
        'forum',
        _l(u'Conversations'),
        condition=lambda ctx: g.community.has_forum),
    CommunityTabAction(
        'communities:tabs',
        'calendar',
        _l(u'Calendar'),
        condition=lambda ctx: g.community.has_calendar),
    CommunityTabAction(
        'communities:tabs',
        'members',
        _l(u'Members'),
        endpoint="communities.members"),
    CommunityTabAction(
        'communities:tabs',
        'settings',
        _l(u'Settings'),
        icon='cog',
        condition=lambda ctx: current_user.has_role("admin"),
        endpoint="communities.settings"))


def register_actions(state):
    if not actions.installed(state.app):
        return
    with state.app.app_context():
        actions.register(*_actions)



"""
Not used!
"""

from __future__ import absolute_import, print_function

from flask import flash, g, redirect, render_template, request, url_for
from flask_babel import gettext as _
from flask_mail import Message as Email
from sqlalchemy.schema import Column
from sqlalchemy.types import Text

from abilian.core.extensions import db, mail

from .social import social


class Invite(db.Model):
    """Tracks a pending invitation to join the community.
    """

    __tablename__ = 'invitation'

    email = Column(Text, nullable=False)

    query = db.session.query_property()


@social.route("/invite")
def invite():
    return render_template("social/invite.html")


@social.route("/invite", methods=['POST'])
def invite_post():
    action = request.form.get("action", "cancel")
    if action == 'cancel':
        flash(_(u"Action aborted"), "info")
        return redirect(url_for(".home"))

    emails = request.form.get("emails").split("\n")
    emails = [email.strip() for email in emails]

    # FIXME: what do we do with this ?
    message = request.form.get("message", "")  # noqa

    with mail.connect() as conn:
        for email in emails:
            # FIXME: what do we do with this ?
            invite = Invite(sender=g.user, email=email)  # noqa

            # subject = _(u"%s would like to invite you to the %s community") % (g.user.name, "Yaka")
            subject = "%s would like to invite you to the %s community" % (
                g.user.name, "Yaka")
            msg = Email(subject, recipients=[email], sender=g.user.email)
            params = dict(org_name="Yaka")
            msg.body = render_template('social/mail/invite.txt', **params)
            conn.send(msg)

    flash(_(u"Invitation sent"), "info")
    return redirect(url_for(".home"))



# coding=utf-8
"""
Configuration and injectable fixtures for Pytest.

Supposed to replace the too-complex current UnitTest-based testing
framework.

DI and functions over complex inheritance hierarchies FTW!
"""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

from pytest import fixture, yield_fixture

from abilian.sbe.app import create_app
from abilian.services import get_service


class TestConfig:
    TESTING = True
    SERVER_NAME = 'localhost'
    # CSRF_ENABLED = False


@fixture(scope='session')
def app():
    """We usually only create an app once per session."""
    _app = create_app(config=TestConfig)
    return _app


@yield_fixture
def db(app):
    """Return a fresh db for each test."""
    from abilian.core.extensions import db

    with app.app_context():
        stop_all_services(app)
        ensure_services_started(['repository', 'session_repository'])

        cleanup_db(db)
        db.create_all()
        yield db

        db.session.remove()
        cleanup_db(db)
        stop_all_services(app)


@fixture
def client(app, db):
    """Return a Web client, used for testing, bound to a DB session."""
    return app.test_client()


#
# Cleanup utilities
#
def cleanup_db(db):
    """Drop all the tables, in a way that doesn't raise integrity errors."""

    # Need to run this sequence twice for some reason
    for i in range(0, 2):
        for table in reversed(db.metadata.sorted_tables):
            try:
                db.session.execute(table.delete())
            except:
                pass
    # Just in case ?
    db.drop_all()


def stop_all_services(app):
    for service in app.services.values():
        if service.running:
            service.stop()


def ensure_services_started(services):
    for service_name in services:
        service = get_service(service_name)
        if not service.running:
            service.start()



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function, unicode_literals

from flask import url_for

from abilian.services.security import Admin

from .. import views
from ..models import Community
from .base import CommunityIndexingTestCase as BaseIndexingTestCase


class CommunityIndexingTestCase(BaseIndexingTestCase):

    def test_community_indexed(self):
        svc = self.svc
        obj_types = (Community.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search('community', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search('community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == self.community.object_key

        with self.login(self.user_c2):
            res = svc.search('community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == self.c2.object_key

    def test_default_view_kw_with_hit(self):
        with self.login(self.user):
            hit = self.svc.search(
                'community', object_types=(Community.entity_type,))[0]
            kw = views.default_view_kw({}, hit, hit['object_type'], hit['id'])

        assert kw == {'community_id': self.community.slug}


class CommunityWebTestCase(BaseIndexingTestCase):
    # FIXME later
    SQLALCHEMY_WARNINGS_AS_ERROR = False

    def test_index(self):
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url_for("communities.index"))
        self.assert_200(response)

    def test_community_home(self):
        url = self.app.default_view.url_for(self.community)
        user = self.user.email
        user_c2 = self.user_c2.email
        with self.client_login(user_c2, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 403

        with self.client_login(user, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 302
            expected_url = url_for(
                "wall.index", community_id=self.community.slug, _external=True)
            assert response.headers['Location'] == expected_url

    def test_community_settings(self):
        url = url_for('communities.settings', community_id=self.community.slug)
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            response = self.client.get(url)
            self.assert_200(response)

            data = {
                '__action': 'edit',
                'name': 'edited community',
                'description': 'my community',
                'linked_group': '',
                'type': 'participative',
            }
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            assert response.headers['Location'] == \
                   'http://localhost/communities/{}/'.format(self.community.slug)

            community = Community.query.get(self.community.id)
            assert community.name == 'edited community'

    def test_new(self):
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url_for("communities.new"))
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            response = self.client.get(url_for("communities.new"))
            assert response.status_code == 200

    def test_members(self):
        with self.client_login(self.user.email, 'azerty'):
            url = url_for(
                "communities.members", community_id=self.community.slug)
            response = self.client.get(url)
            self.assert_200(response)

            # test add user
            data = {'action': 'add-user-role', 'user': self.user_c2.id}
            response = self.client.post(url, data=data)
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            data = {
                'action': 'add-user-role',
                'user': self.user_c2.id,
                'role': 'member',
            }
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            assert response.headers['Location'] == 'http://localhost' + url

            membership = [
                m for m in self.community.memberships if m.user == self.user_c2
            ][0]
            assert membership.role == 'member'

            data['action'] = 'set-user-role'
            data['role'] = 'manager'
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            assert response.headers['Location'] == 'http://localhost' + url

            self.session.expire(membership)
            assert membership.role == 'manager'

            # Community.query.session is not self.db.session, but web app session.
            community = Community.query.get(self.community.id)
            assert self.user_c2 in community.members

            # test delete
            data = {
                'action':
                'delete',
                'user':
                self.user_c2.id,
                'membership':
                [m.id for m in community.memberships
                 if m.user == self.user_c2][0],
            }
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            assert response.headers['Location'] == \
                   'http://localhost/communities/{}/members'.format(
                       self.community.slug)

            assert self.user_c2 not in community.members



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

from typing import Any, Dict

from flask import url_for as url_for_orig
from flask import g

from abilian.i18n import _l
from abilian.sbe.apps.communities.security import is_manager
from abilian.services.security import MANAGE, WRITE, security
from abilian.web.action import Action, FAIcon, ModalActionMixin, actions

from .repository import repository


def url_for(endpoint, **kw):
    return url_for_orig(endpoint, community_id=g.community.slug, **kw)


class CmisContentAction(Action):
    sbe_type = None  # type: str
    permission = None  # type: str

    def __init__(self, *args, **kwargs):
        if 'permission' in kwargs:
            self.permission = kwargs.pop('permission')

        Action.__init__(self, *args, **kwargs)

    def pre_condition(self, ctx):
        # type: (Dict[str, Any]) -> bool
        obj = ctx['object']
        ok = obj.sbe_type == self.sbe_type

        if ok and self.permission is not None:
            ok = self.has_access(self.permission, obj)

        return ok

    def has_access(self, permission, obj):
        # type: (str, Any) -> bool
        return repository.has_permission(g.user, permission, obj)


class BaseFolderAction(CmisContentAction):
    """Apply to all folders, including root folder.
    """
    sbe_type = u'cmis:folder'


class FolderButtonAction(BaseFolderAction):

    _std_template_string = (
        u'<button class="btn {{ action.css_class }}" name="action" '
        u'value="{{ action.name }}" title="{{ action.title }}">'
        u'{{ action.icon }}</button>')

    _modal_template_string = (
        u'<a class="btn {{ action.css_class }}" href="{{ url }}" '
        u'data-toggle="modal" role="button" title="{{ action.title }}">'
        u'{{ action.icon }}</a>')

    def __init__(self, *args, **kwargs):
        self.modal = False

        if 'modal' in kwargs:
            self.modal = kwargs.pop('modal')

        css_class = kwargs.pop('css_class', u'btn-default')
        self.CSS_CLASS = self.CSS_CLASS + u' ' + css_class

        BaseFolderAction.__init__(self, *args, **kwargs)

    @property
    def template_string(self):
        return (self._modal_template_string
                if self.modal else self._std_template_string)


class FolderAction(BaseFolderAction):
    """Apply to all folders except root folder.
    """
    sbe_type = u'cmis:folder'

    def pre_condition(self, ctx):
        return (super(FolderAction, self).pre_condition(ctx) and
                ctx['object'] is not repository.root_folder)


class FolderPermisionsAction(BaseFolderAction):
    """Apply to all folders except root folder.
    """
    sbe_type = u'cmis:folder'

    def pre_condition(self, ctx):
        return (super(BaseFolderAction, self).pre_condition(ctx) and
                ctx['object'].depth > 1)


class FolderModalAction(ModalActionMixin, FolderAction):
    pass


class DocumentAction(CmisContentAction):
    sbe_type = u'cmis:document'


class DocumentModalAction(ModalActionMixin, DocumentAction):
    pass


class RootFolderAction(CmisContentAction):
    """Apply only for root folder."""

    def pre_condition(self, ctx):
        return ctx['object'] is repository.root_folder


_checkin_template_action = u'''
<form method="POST" action="{{ url }}" encoding="multipart/form-data" target="_new">
  {{ csrf.field() }}
  <button type="submit" class="btn btn-link" name="action"
          value="{{ action.name}}"
     onclick="window.setTimeout(function () { window.location.reload() }, 500)">
    {%- if action.icon %}{{ action.icon }} {% endif %}
    {{ action.title }}
  </button>
</form>
'''

_lock_template_action = u'''
<form method="POST" action="{{ url }}" encoding="multipart/form-data">
  {{ csrf.field() }}
  <button type="submit" class="btn btn-link" name="action"
          value="{{ action.name}}">
    {%- if action.icon %}{{ action.icon }} {% endif %}
    {{ action.title }}
  </button>
</form>
'''

_actions = (
    # Folder listing action buttons ##########
    FolderButtonAction(
        'documents:folder-listing',
        'download',
        _l(u'Download'),
        icon='download'),
    FolderButtonAction(
        'documents:folder-listing',
        'move-files',
        _l(u'Move to another folder'),
        icon='move',
        url='#modal-move-files',
        modal=True,
        permission=WRITE),
    FolderButtonAction(
        'documents:folder-listing',
        'delete',
        _l(u'Delete'),
        permission=WRITE,
        icon='trash',
        css_class='btn-danger'),
    FolderButtonAction(
        'documents:folder-listing',
        'change-owner',
        _l(u'Change owner'),
        icon='user',
        url='#modal-change-owner',
        modal=True,
        permission=MANAGE),
    # Folder left bar actions ##########
    # view
    RootFolderAction(
        'documents:content',
        'view',
        _l(u'List content'),
        icon='list',
        condition=(lambda ctx: security.has_role(g.user, "admin")),
        url=lambda ctx: url_for(".folder_view", folder_id=ctx['object'].id),
    ),
    # view
    FolderAction(
        'documents:content',
        'view',
        _l(u'List content'),
        icon='list',
        url=lambda ctx: url_for(".folder_view", folder_id=ctx['object'].id),
    ),
    # Descendants
    FolderAction(
        'documents:content',
        'descendants',
        _l(u'View descendants'),
        icon=FAIcon('code-fork fa-rotate-90'),
        url=
        lambda ctx: url_for(".descendants_view", folder_id=ctx['object'].id),
    ),
    # upload
    FolderModalAction(
        'documents:content',
        'upload_files',
        _l('Upload file(s)'),
        icon='upload',
        url='#modal-upload-files',
        permission=WRITE),
    # edit
    FolderModalAction(
        'documents:content',
        'edit',
        _l('Edit properties'),
        icon='pencil',
        url='#modal-edit',
        permission=WRITE),
    # new folder
    FolderModalAction(
        'documents:content',
        'new_folder',
        _l('New folder'),
        icon='plus',
        url='#modal-new-folder',
        permission=WRITE),

    # # members
    # FolderAction(
    #   'documents:content', 'members', _l('Members'), icon='user',
    #   url=lambda ctx: url_for(".members", folder_id=ctx['object'].id)),

    # permissions
    FolderPermisionsAction(
        'documents:content',
        'permissions',
        _l('Permissions'),
        icon='lock',
        url=lambda ctx: url_for(".permissions", folder_id=ctx['object'].id),
        permission=MANAGE),

    # Document actions ##########
    # View / preview in browser
    DocumentAction(
        'documents:content',
        'preview',
        _l(u'View in browser'),
        icon='eye-open',
        url=lambda ctx: url_for('.document_preview', doc_id=ctx['object'].id),
        condition=
        (lambda ctx: ctx['object'].antivirus_ok and ctx['object'].content_type in ('text/html', 'text/plain', 'application/pdf')
        )),
    # viewers
    DocumentModalAction(
        'documents:content',
        'document_viewers',
        _l(u'Viewers list'),
        icon='user',
        condition=lambda ctx: is_manager(context=ctx),
        url=lambda ctx: url_for(".document_viewers", doc_id=ctx['object'].id)),
    # edit
    DocumentModalAction(
        'documents:content',
        'edit',
        _l(u'Edit properties'),
        icon='pencil',
        url='#modal-edit',
        permission=WRITE),
    # Checkin / Checkout
    DocumentAction(
        'documents:content',
        'checkout',
        _l(u'Checkout (Download for edit)'),
        icon='download',
        url=lambda ctx: url_for('.checkin_checkout', doc_id=ctx['object'].id),
        condition=lambda ctx: ctx['object'].lock is None,
        template_string=_checkin_template_action),
    # DocumentAction(
    #   'documents:content', 'lock', _l(u'Lock for edit'),
    #   icon='lock',
    #   url=lambda ctx: url_for('.checkin_checkout', doc_id=ctx['object'].id),
    #   condition=lambda ctx: ctx['object'].lock is None,
    #   template_string=_lock_template_action,
    # ),
    DocumentAction(
        'documents:content',
        'unlock',
        _l(u'Unlock'),
        icon=FAIcon('unlock'),
        url=lambda ctx: url_for('.checkin_checkout', doc_id=ctx['object'].id),
        condition=lambda ctx: ctx['object'].lock is not None,
        template_string=_lock_template_action),
    # upload-new / checkin
    DocumentModalAction(
        'documents:content',
        'upload',
        _l(u'Upload new version'),
        icon='upload',
        url='#modal-upload-new-version',
        # either not locked, either user is owner
        condition=
        lambda ctx: not ctx['object'].lock or ctx['object'].lock.is_owner(),
        permission=WRITE),
    # send by email
    DocumentModalAction(
        'documents:content',
        'send_by_email',
        _l(u'Send by email'),
        icon='envelope',
        url='#modal-send-by-email',
        condition=lambda ctx: ctx['object'].antivirus_ok,
        permission=WRITE),
    # delete
    DocumentModalAction(
        'documents:content',
        'delete',
        _l(u'Delete'),
        icon='trash',
        url='#modal-delete',
        permission=WRITE),
    # refresh preview
    DocumentAction(
        'documents:content',
        'refresh_preview',
        _l(u'Refresh preview'),
        icon='refresh',
        url=lambda ctx: url_for('.refresh_preview', doc_id=ctx['object'].id),
        condition=lambda ctx: ctx['object'].antivirus_ok,
        permission=MANAGE),
)


def register_actions(state):
    if not actions.installed(state.app):
        return

    with state.app.app_context():
        actions.register(*_actions)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

from flask import render_template

from abilian.core.models.subjects import User
from abilian.sbe.apps.communities.models import WRITER
from abilian.sbe.apps.communities.tests.base import CommunityBaseTestCase
from abilian.sbe.apps.notifications.tasks.social import CommunityDigest, \
    generate_unsubscribe_token
from abilian.sbe.testing import BaseTestCase
from abilian.web import url_for


class TestNotificationViews(BaseTestCase):

    def get_setup_config(self):
        cfg = super(TestNotificationViews, self).get_setup_config()
        # "unsubscribe" url must be accessible without csrf, so user can post even
        # if he's not authentified
        cfg.CSRF_ENABLED = True
        cfg.WTF_CSRF_ENABLED = True
        cfg.NO_LOGIN = False
        return cfg

    def setUp(self):
        super(TestNotificationViews, self).setUp()
        self.user = User(
            email=u'user_1@example.com', password='azerty', can_login=True)
        self.session.add(self.user)
        self.session.commit()

    def test_unsubscribe(self):
        preferences = self.app.services['preferences']
        preferences.set_preferences(self.user, **{
            'sbe:notifications:daily': True
        })
        token = generate_unsubscribe_token(self.user)
        url = url_for('notifications.unsubscribe_sbe', token=token)

        r = self.client.get(url)
        self.assert_200(r)
        prefs = preferences.get_preferences(self.user)
        self.assertTrue(prefs['sbe:notifications:daily'])

        r = self.client.post(url)
        self.assert_200(r)
        prefs = preferences.get_preferences(self.user)
        self.assertFalse(prefs['sbe:notifications:daily'])


class NotificationTestCase(CommunityBaseTestCase):

    def setUp(self):
        super(NotificationTestCase, self).setUp()
        self.user = User(
            email=u'user_1@example.com', password='azerty', can_login=True)
        self.session.add(self.user)
        self.community.set_membership(self.user, WRITER)
        self.session.commit()

    def test_mail_templates(self):
        # this actually tests that templates are parsed without errors, not the
        # rendered content
        digests = [CommunityDigest(self.community)]
        token = generate_unsubscribe_token(self.user)
        render_template(
            "notifications/daily-social-digest.txt",
            digests=digests,
            token=token)
        render_template(
            "notifications/daily-social-digest.html",
            digests=digests,
            token=token)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function, unicode_literals

import sys
import unittest
from io import BytesIO
from itertools import count
from os.path import dirname, join
from zipfile import ZipFile

import flask_mail
import pytest
from flask import g, get_flashed_messages
from toolz import first
from werkzeug.datastructures import FileStorage

from abilian.sbe.apps.communities.models import WRITER
from abilian.sbe.apps.communities.presenters import CommunityPresenter
from abilian.sbe.apps.communities.tests.base import CommunityBaseTestCase, \
    CommunityIndexingTestCase
from abilian.web.util import url_for

from ..models import Document, Folder, PathAndSecurityIndexable, db
from ..views import util as view_util
from ..views.folders import explore_archive


class BaseTests(CommunityBaseTestCase):
    init_data = True
    no_login = True

    def setUp(self):
        super(BaseTests, self).setUp()

        self.root_folder = Folder(title="root")
        db.session.add(self.root_folder)
        db.session.flush()

    @staticmethod
    def uid_from_url(url):
        return int(url.split("/")[-1])

    @staticmethod
    def open_file(filename):
        path = join(dirname(__file__), "data", "dummy_files", filename)
        return open(path, 'rb')


class TestBlobs(BaseTests):

    def test_document(self):
        root = Folder(title="root")
        doc = Document(parent=root, title="test")
        data = self.open_file("onepage.pdf").read()
        doc.set_content(data, "application/pdf")
        self.session.add(doc)
        self.session.commit()

        # coverage
        self.app.config['ANTIVIRUS_CHECK_REQUIRED'] = True
        doc.ensure_antivirus_scheduled()

    def test_antivirus_properties(self):
        root = Folder(title="root")
        doc = Document(parent=root, title="test")
        doc.set_content(b'content', 'text/plain')
        appcfg = self.app.config
        meta = doc.content_blob.meta

        # 1: not check required ################
        # no data in meta
        appcfg['ANTIVIRUS_CHECK_REQUIRED'] = False
        assert doc.antivirus_scanned is False
        assert doc.antivirus_status is None
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # antivirus was run, but no result
        meta['antivirus'] = None
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is None
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # virus detected
        meta['antivirus'] = False
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is False
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is False

        # virus free
        meta['antivirus'] = True
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is True
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True

        # 2: check required ##################
        # no data in meta
        del meta['antivirus']
        appcfg['ANTIVIRUS_CHECK_REQUIRED'] = True
        assert doc.antivirus_scanned is False
        assert doc.antivirus_status is None
        assert doc.antivirus_required is True
        assert doc.antivirus_ok is False

        # antivirus was run, but no result
        meta['antivirus'] = None
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is None
        assert doc.antivirus_required is True
        assert doc.antivirus_ok is False

        # virus detected
        meta['antivirus'] = False
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is False
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is False

        # virus free
        meta['antivirus'] = True
        assert doc.antivirus_scanned is True
        assert doc.antivirus_status is True
        assert doc.antivirus_required is False
        assert doc.antivirus_ok is True


class IndexingTestCase(CommunityIndexingTestCase):

    def test_folder_indexed(self):
        folder = Folder(title='Folder 1', parent=self.community.folder)
        self.session.add(folder)
        folder_other = Folder(title='Folder 2: other', parent=self.c2.folder)
        self.session.add(folder_other)
        self.session.commit()

        svc = self.svc
        obj_types = (Folder.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search('folder', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search('folder', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == folder.object_key
            assert hit['community_slug'] == self.community.slug

        with self.login(self.user_c2):
            res = svc.search('folder', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == folder_other.object_key
            assert hit['community_slug'] == self.c2.slug


class TestViews(CommunityIndexingTestCase, BaseTests):

    def setUp(self):
        super(TestViews, self).setUp()
        self.community.type = 'participative'
        self.community.set_membership(self.user, WRITER)
        self.session.commit()
        self.folder = self.community.folder

    def test_util_create(self):
        with self.login(self.user):
            g.user = self.user
            g.community = CommunityPresenter(self.community)
            name = 'document'
            fs = FileStorage(
                BytesIO(b'content'), filename=name, content_type='text/plain')
            doc = view_util.create_document(self.folder, fs)
            self.session.flush()
            assert doc.parent == self.folder
            assert doc.name == name

            # test upload with same name: should be renamed
            fs = FileStorage(
                BytesIO(b'content'), filename=name, content_type='text/plain')
            doc2 = view_util.create_document(self.folder, fs)
            self.session.flush()
            assert doc2.parent == self.folder
            assert len(self.folder.children) == 2
            assert doc2.name == name + '-1'

            messages = get_flashed_messages()
            assert len(messages) == 1

    def test_home(self):
        with self.client_login(self.user.email, password='azerty'):
            response = self.get(
                url_for('documents.index', community_id=self.community.slug))
            self.assert_status(response, 302)
            self.assertEqual(
                response.headers['Location'],
                'http://localhost/communities/{}/docs/folder/{}'
                ''.format(self.community.slug, self.folder.id),
            )

    def _test_upload(self,
                     title,
                     content_type,
                     test_preview=True,
                     assert_preview_available=True):
        data = {
            'file': (self.open_file(title), title, content_type),
            'action': 'upload',
        }

        folder = self.community.folder
        url = url_for(
            "documents.folder_post",
            community_id=self.community.slug,
            folder_id=folder.id)
        response = self.client.post(url, data=data)
        self.assert_status(response, 302)

        doc = folder.children[0]
        assert doc.title == title
        url = url_for(
            "documents.document_view",
            community_id=self.community.slug,
            doc_id=doc.id)
        response = self.get(url)
        self.assert_200(response)

        url = url_for(
            "documents.document_download",
            community_id=self.community.slug,
            doc_id=doc.id)
        response = self.get(url)
        self.assert_200(response)
        assert response.headers['Content-Type'] == content_type

        content = self.open_file(title).read()
        assert response.data == content

        if test_preview:
            url = url_for(
                "documents.document_preview_image",
                community_id=self.community.slug,
                doc_id=doc.id,
                size=500)
            response = self.get(url)
            if assert_preview_available:
                self.assert_200(response)
                assert response.headers['Content-Type'] == 'image/jpeg'
            else:
                # redirect to 'missing image'
                self.assert_302(response)
                assert response.headers['Cache-Control'] == 'no-cache'

        url = url_for(
            "documents.document_delete",
            community_id=self.community.slug,
            doc_id=doc.id)
        response = self.client.post(url)
        self.assert_302(response)

        url = url_for(
            "documents.document_view",
            community_id=self.community.slug,
            doc_id=doc.id)
        response = self.get(url)
        self.assert_404(response)

    def test_text_upload(self):
        name = 'wikipedia-fr.txt'
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(name, "text/plain", test_preview=False)

    def test_pdf_upload(self):
        name = "onepage.pdf"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(name, "application/pdf")

    def test_image_upload(self):
        NAME = "picture.jpg"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(NAME, "image/jpeg")

    @pytest.mark.skip()  # FIXME: magic detection mismatch?
    def test_binary_upload(self):
        NAME = "random.bin"
        with self.client_login(self.user.email, password='azerty'):
            self._test_upload(
                NAME,
                "application/octet-stream",
                assert_preview_available=False)

    @pytest.mark.skipif(
        sys.version_info >= (3, 0), reason="Doesn't work yet on Py3k")
    def test_explore_archive(self):

        fd = self.open_file('content.zip')
        result = [('/'.join(path), f) for path, f in explore_archive(fd)]
        assert result == [('', fd)]

        fd = self.open_file('content.zip')
        archive_content = explore_archive(fd, uncompress=True)
        result = {
            '/'.join(path) + '/' + f.filename
            for path, f in archive_content
        }
        assert result == {
            'existing-doc/file.txt',
            'existing-doc/subfolder_in_renamed/doc.txt',
            'folder 1/doc.txt',
            'folder 1/dos cp437: é.txt',
            'folder 1/osx: utf-8: é.txt',
        }

    def test_zip_upload_uncompress(self):
        folder = Folder(title='folder 1', parent=self.community.folder)
        self.session.add(folder)
        self.session.flush()
        folder = self.community.folder
        files = []
        files.append((BytesIO(b'A document'), 'existing-doc', 'text/plain'))
        files.append((self.open_file('content.zip'), 'content.zip',
                      'application/zip'))
        data = {'file': files, 'action': 'upload', 'uncompress_files': True}
        url = url_for(
            "documents.folder_post",
            community_id=self.community.slug,
            folder_id=folder.id)
        with self.client_login(self.user.email, password='azerty'):
            response = self.client.post(url, data=data)

        assert response.status_code == 302
        expected = {'existing-doc', 'folder 1', 'existing-doc-1'}
        assert expected == {f.title for f in folder.children}
        expected = {'folder 1', 'existing-doc-1'}
        assert expected == {f.title for f in folder.subfolders}

    def test_zip(self):
        with self.client_login(self.user.email, password='azerty'):
            title = "onepage.pdf"
            content_type = "application/pdf"
            data = {
                'file': (self.open_file(title), title, content_type),
                'action': 'upload',
            }
            folder = self.community.folder
            url = url_for(
                "documents.folder_post",
                community_id=self.community.slug,
                folder_id=folder.id)
            response = self.client.post(url, data=data)
            assert response.status_code == 302

            doc = folder.children[0]
            data = {
                'action': 'download',
                'object-selected': ["cmis:document:%d" % doc.id],
            }
            url = url_for(
                "documents.folder_post",
                community_id=self.community.slug,
                folder_id=folder.id)
            response = self.client.post(url, data=data)
            assert response.status_code == 200
            assert response.content_type == 'application/zip'

            zipfile = ZipFile(BytesIO(response.data))
            assert zipfile.namelist() == [title]

    def test_recursive_zip(self):
        with self.client_login(self.user.email, password='azerty'):
            data = {
                'action': 'new',
                'title': 'my folder',
            }
            folder = self.community.folder
            url = url_for(
                "documents.folder_post",
                community_id=self.community.slug,
                folder_id=folder.id)
            response = self.client.post(url, data=data)
            assert response.status_code == 302

            my_folder = folder.children[0]

            title = "onepage.pdf"
            content_type = "application/pdf"
            data = {
                'file': (self.open_file(title), title, content_type),
                'action': 'upload',
            }
            url = url_for(
                "documents.folder_post",
                community_id=self.community.slug,
                folder_id=my_folder.id)
            response = self.client.post(url, data=data)
            assert response.status_code == 302

            data = {
                'action': 'download',
                'object-selected': ["cmis:folder:%d" % my_folder.id],
            }
            url = url_for(
                "documents.folder_post",
                community_id=self.community.slug,
                folder_id=folder.id)
            response = self.client.post(url, data=data)
            assert response.status_code == 200
            assert response.content_type == 'application/zip'

            zipfile = ZipFile(BytesIO(response.data))
            assert zipfile.namelist() == ['my folder/' + title]

    def test_document_send_by_mail(self):
        mail = self.app.extensions['mail']
        folder = self.community.folder

        with self.client_login(self.user.email, password='azerty'):
            # upload files
            for filename in ('ascii title.txt', 'utf-8 est arrivé!.txt'):
                content_type = "text/plain"
                data = {
                    'file': (BytesIO(b'file content'), filename, content_type),
                    'action': 'upload',
                }
                url = url_for(
                    "documents.folder_post",
                    community_id=self.community.slug,
                    folder_id=folder.id)
                self.client.post(url, data=data)

            ascii_doc = folder.children[0]
            unicode_doc = folder.children[1]

            def get_send_url(doc_id):
                return url_for(
                    'documents.document_send',
                    community_id=self.community.slug,
                    doc_id=doc_id)

            # mail ascii filename
            with mail.record_messages() as outbox:
                url = get_send_url(ascii_doc.id)
                rv = self.client.post(
                    url,
                    data={
                        'recipient': 'dest@example.com',
                        'message': 'Voilà un fichier',
                    })
                self.assertEqual(rv.status_code, 302,
                                 "expected 302, got:" + rv.status)
                assert len(outbox) == 1

                msg = outbox[0]
                assert msg.subject == '[Abilian Test] Unknown sent you a file'
                assert msg.recipients == ['dest@example.com']

                assert len(msg.attachments) == 1

                attachment = first(msg.attachments)
                assert isinstance(attachment, flask_mail.Attachment)
                assert attachment.filename == "ascii title.txt"

            # mail unicode filename
            with mail.record_messages() as outbox:
                url = get_send_url(unicode_doc.id)
                rv = self.client.post(
                    url,
                    data={
                        'recipient': 'dest@example.com',
                        'message': 'Voilà un fichier',
                    })
                assert rv.status_code == 302
                assert len(outbox) == 1

                msg = outbox[0]
                assert isinstance(msg, flask_mail.Message)
                assert msg.subject == '[Abilian Test] Unknown sent you a file'
                assert msg.recipients == ['dest@example.com']
                assert len(msg.attachments) == 1

                attachment = first(msg.attachments)
                assert isinstance(attachment, flask_mail.Attachment)
                assert attachment.filename == "utf-8 est arrivé!.txt"


class TestPathIndexable(unittest.TestCase):

    class MockPath(PathAndSecurityIndexable):

        def __init__(self, id, parent=None):
            self.id = id
            self.parent = parent

    def setUp(self):
        id_gen = count()
        obj = self.MockPath(next(id_gen))
        obj = self.MockPath(next(id_gen), parent=obj)
        obj = self.MockPath(next(id_gen), parent=obj)
        self.obj = self.MockPath(next(id_gen), parent=obj)

    def test_iter_to_root(self):
        assert [o.id for o in self.obj._iter_to_root()] == [3, 2, 1, 0]
        assert [o.id
                for o in self.obj._iter_to_root(skip_self=True)] == [2, 1, 0]

    def test_indexable_parent_ids(self):
        self.assertEqual(self.obj._indexable_parent_ids, '/0/1/2')



# coding=utf-8
"""
Static configuration for the application.

TODO: add more (runtime) flexibility in plugin discovery, selection
and activation.
"""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

import logging
import os
import subprocess
import sys
from pathlib import Path

import jinja2
from flask import current_app
from flask_script import Command, Manager
from six import text_type
from werkzeug.serving import BaseWSGIServer

from abilian.app import Application as BaseApplication
from abilian.core.celery import FlaskCelery as BaseCelery
from abilian.core.celery import FlaskLoader as CeleryBaseLoader
from abilian.core.commands import setup_abilian_commands
from abilian.core.extensions import db
from abilian.services import converter

from .apps.documents.repository import repository
from .extension import sbe

# Used for side effects, do not remove

__all__ = ['create_app', 'db']

logger = logging.getLogger(__name__)


def create_app(config=None):
    return Application(config=config)


command_manager = Manager(create_app)
setup_abilian_commands(command_manager)


# loader to be used by celery workers
class CeleryLoader(CeleryBaseLoader):
    flask_app_factory = 'abilian.sbe.app.create_app'


celery = BaseCelery(loader=CeleryLoader)


class Application(BaseApplication):

    APP_PLUGINS = BaseApplication.APP_PLUGINS + (
        "abilian.sbe.apps.main",
        "abilian.sbe.apps.notifications",
        "abilian.sbe.apps.preferences",
        "abilian.sbe.apps.wiki",
        "abilian.sbe.apps.wall",
        "abilian.sbe.apps.documents",
        "abilian.sbe.apps.forum",
        # "abilian.sbe.apps.calendar",
        "abilian.sbe.apps.communities",
        "abilian.sbe.apps.social",
        "abilian.sbe.apps.preferences",
    )

    script_manager = command_manager

    def __init__(self, name='abilian_sbe', config=None, **kwargs):
        BaseApplication.__init__(self, name, config=config, **kwargs)
        loader = jinja2.PackageLoader('abilian.sbe', 'templates')
        self.register_jinja_loaders(loader)

    def init_extensions(self):
        BaseApplication.init_extensions(self)
        sbe.init_app(self)
        repository.init_app(self)
        converter.init_app(self)


# SBE demo app bootstrap stuff
_SBE_DEMO_SCRIPT = '''\
#!{BIN_DIR}/python
from __future__ import absolute_import

import sys
from abilian.sbe.app import command_entry_point

sys.exit(command_entry_point())
'''

_BASE_SERVER_ACTIVATE = BaseWSGIServer.server_activate


def _on_http_server_activate(self, *args, **kwargs):
    """
    This function is used as to monkey patch BaseWSGIServer.server_activate
    during `setup_sbe_demo`.
    """
    _BASE_SERVER_ACTIVATE(self, *args, **kwargs)
    # now we are listening to socket
    host, port = self.server_address
    if host == '0.0.0.0':
        # chrome is not ok with 0.0.0.0
        host = 'localhost'
    url = 'http://{host}:{port}/setup'.format(host=host, port=port)

    if sys.platform == "win32":
        os.startfile(url)
    else:
        opener = "open" if sys.platform == "darwin" else "xdg-open"
        subprocess.call([opener, url])


# run with python -m abilian.sbe.app setup_sbe_app
def setup_sbe_app():
    """
    Basic set up SBE application. Must be run inside a virtualenv.

    Will create `abilian_sbe` script, run a local server and open browser on app's
    setup wizard.
    """
    logger = logging.getLogger('sbe_demo')
    logger.setLevel(logging.INFO)

    if 'VIRTUAL_ENV' not in os.environ:
        logger.error('Not in a virtualenv! Aborting.')
        return 1

    bin_dir = Path(sys.prefix) / 'bin'

    if not bin_dir.exists() and bin_dir.is_dir():
        logger.error('%s doesn\'t exists or is not a directory. Aborting',
                     repr(text_type(bin_dir)))
        return 1

    script_file = bin_dir / 'abilian_sbe'
    if script_file.exists():
        logger.info('%s already exists. Skipping creation.',
                    repr(text_type(script_file)))
    else:
        with script_file.open('w') as out:
            logger.info('Create script: "%s".', repr(text_type(script_file)))
            content = _SBE_DEMO_SCRIPT.format(BIN_DIR=text_type(bin_dir))
            out.write(content)
        script_file.chmod(0o755)  # 0755: -rwxr-xr-x

    current_app.config['PRODUCTION'] = True
    current_app.config['DEBUG'] = False
    current_app.config['ASSETS_DEBUG'] = False
    current_app.config['SITE_NAME'] = 'Abilian SBE'
    current_app.config['MAIL_SENDER'] = 'abilian-sbe-app@example.com'

    logger.info('Prepare CSS & JS files')
    command_manager.handle('abilian_sbe', ['assets', 'build'])
    # disabled init config: only if not running setupwizard
    # command_manager.handle('abilian_sbe', ['config', 'init'])

    # patch server used to launch browser here immediately after socket opened
    BaseWSGIServer.server_activate = _on_http_server_activate
    return command_manager.handle('abilian_sbe', ['run', '--hide-config'])


def command_entry_point():
    command_manager.run(commands={'setup_sbe_app': Command(setup_sbe_app)},)


if __name__ == '__main__':
    command_entry_point()



# coding=utf-8
from __future__ import absolute_import, print_function, unicode_literals

from os.path import dirname, join

from flask import current_app, flash, g, jsonify, make_response, redirect, \
    render_template, request, url_for
from flask_babel import gettext as _
from sqlalchemy import func
from werkzeug.exceptions import InternalServerError, NotFound

from abilian.core.extensions import db
from abilian.core.models.subjects import Group, User
from abilian.services import get_service
from abilian.services.image import CROP, resize
from abilian.services.security import security
from abilian.web import csrf
from abilian.web.views import default_view

from .social import social
from .util import Env

DEFAULT_GROUP_MUGSHOT = open(
    join(dirname(__file__), "../../../static/images/frog.jpg"), 'rb').read()


@social.route("/groups/")
def groups():
    tab = request.args.get("tab", "all_groups")
    e = Env()
    if tab == 'all_groups':
        e.groups = Group.query.order_by(Group.name).all()
        if not security.has_role(g.user, "admin"):
            e.groups = [
                group for group in e.groups
                if group.public or g.user in group.members
            ]
    else:
        e.groups = g.user.groups
        e.groups.sort(key=lambda x: x.name)
    return render_template("social/groups.html", **e)


def is_admin(group):
    security = get_service('security')
    is_admin = g.user in group.admins
    if not is_admin and 'security' in current_app.extensions:
        is_admin = security.has_role(g.user, 'admin')

    return is_admin


@social.route("/groups/<int:group_id>")
@default_view(social, Group, 'group_id')
def group_home(group_id):
    e = Env(csrf_token=csrf.field())
    e.group = Group.query.get(group_id)
    e.is_admin = is_admin(e.group)
    return render_template("social/group.html", **e)


@social.route("/groups/<int:group_id>/json")
def group_json(group_id):
    members = Group.query.get(group_id).members
    q = request.args.get("q", '').lower()
    if q:
        members = [
            u for u in members
            if any(
                term.startswith(q) for name in (u.first_name.lower(
                ), u.last_name.lower()) for term in name.split())
        ]

    result = {'results': [{'id': obj.id, 'text': obj.name} for obj in members]}
    return jsonify(result)


@social.route("/groups/<int:group_id>", methods=['POST'])
@csrf.protect
def group_post(group_id):
    group = Group.query.get(group_id)
    action = request.form.get('action')
    return_url = request.form.get('return_url')

    membership_actions = frozenset((
        'add',
        'remove',
        'add-admin',
        'remove-admin',
    ))

    if action not in ('join', 'leave'):
        assert is_admin(group)

    if action == 'join':
        g.user.join(group)
    elif action == 'leave':
        g.user.leave(group)
    elif action in membership_actions:
        user_id = request.form.get('user', '').strip()
        try:
            user_id = int(user_id)
        except:
            flash(_('Error: No user selected'), 'error')
            return redirect(url_for(".group_home", group_id=group_id))

        user = User.query.get(user_id)

        if action == 'add':
            user.join(group)
        elif action == 'remove':
            user.leave(group)
        elif action == 'add-admin':
            if user not in group.admins:
                group.admins.append(user)
        elif action == 'remove-admin':
            if user in group.admins:
                group.admins.remove(user)
    else:
        raise Exception("Should not happen")
    db.session.commit()

    if return_url:
        # TODO: security check
        return redirect(return_url)
    else:
        return redirect(url_for(".group_home", group_id=group_id))


@social.route("/groups/new")
def groups_new():
    # TODO later
    return

    # e = Env()
    # e.form = GroupForm()
    # return render_template("social/groups-new.html", **e)


@social.route("/groups/new", methods=['POST'])
def groups_new_post():
    # TODO later
    return

    # form = GroupForm()
    #
    # if form.validate():
    #     group = Group()
    #     form.populate_obj(group)
    #     db.session.add(group)
    #     db.session.commit()
    #     flash(_(u"Your new group has been created"), category='info')
    #     return redirect(url_for('.group_home', group_id=group.id))
    #
    # else:
    #     e = Env()
    #     e.form = form
    #     return render_template("social/groups-new.html", **e)


@social.route("/groups/<int:group_id>/mugshot")
def group_mugshot(group_id):
    # TODO: duplicated code (with user_mugshot). Extract common method.
    size = int(request.args.get('s', 55))
    if size > 500:
        raise ValueError("Error, size = %d" % size)
    group = Group.query.get(group_id)

    if not group:
        raise NotFound()

    if group.photo:
        data = group.photo
    else:
        data = DEFAULT_GROUP_MUGSHOT

    if size:
        data = resize(data, size, size, mode=CROP)

    response = make_response(data)
    response.headers['content-type'] = 'image/jpeg'
    response.headers.add('Cache-Control', 'public, max-age=600')
    return response


#
# Ad-hoc JSON endpoints, used by select boxes
#
@social.route("/groups/json")
def groups_json():
    q = request.args.get("q").replace("%", " ").lower()

    if not q or len(q) < 2:
        raise InternalServerError()

    query = Group.query
    # query = query.filter(func.lower(Group.name).like(q + "%"))
    query = query.order_by(func.lower(Group.name))
    all = query.all()

    result = {'results': [{'id': obj.id, 'text': obj.name} for obj in all]}
    return jsonify(result)



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function

from distutils.command.build import build as _build

import pip
import setuptools
from setuptools.command.develop import develop as _develop
from setuptools.command.sdist import sdist as _sdist

session = pip.download.PipSession()

_install_requires = pip.req.parse_requirements(
    'requirements.in', session=session)
install_requires = [str(ir.req) for ir in _install_requires]

_dev_requires = pip.req.parse_requirements(
    'etc/dev-requirements.txt', session=session)
dev_requires = [str(ir.req) for ir in _dev_requires]

LONG_DESCRIPTION = open('README.rst', 'r').read()


class build(_build):
    sub_commands = [('compile_catalog', None)] + _build.sub_commands


class sdist(_sdist):
    sub_commands = [('compile_catalog', None)] + _sdist.sub_commands


class develop(_develop):

    def run(self):
        _develop.run(self)
        self.run_command('compile_catalog')


setuptools.setup(
    name='abilian-sbe',
    use_scm_version=True,
    url='https://github.com/abilian/abilian-sbe',
    license='LGPL',
    author='Abilian SAS',
    author_email='contact@abilian.com',
    description=
    'Social Business platform, including: document management, wiki, forum, enterprise social networking, and more',
    long_description=LONG_DESCRIPTION,
    packages=['abilian.sbe'],
    zip_safe=False,
    platforms='any',
    setup_requires=['babel', 'setuptools-git', 'setuptools_scm>=1.5.5'],
    install_requires=install_requires,
    extras_require={
        'tests': dev_requires,
        'dev': dev_requires,
    },
    # dependency_links=dependency_links,
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Flask',
    ],
    cmdclass={
        'build': build,
        'sdist': sdist,
        'develop': develop,
    },
)
